﻿/** 优先加载，最高优先级*/
var LoadPriority_Prior = LoadPriority_Prior || 1;
/** 普通加载*/
var LoadPriority_Common = LoadPriority_Common || 2;
/** 预加载，最低优先级*/
var LoadPriority_Preload = LoadPriority_Preload || 3;

/**
 * 资源加载管理器
 * 对象池，加载优先级，同时加载数量限制，资源缓存，自动释放，加载限制
 * 
 * 若加载对象有属性bDispose，可自动判断跳过加载
 * 该资源加载管理器传入target对象必须严格使用bDispose，不然会导致内存泄露！！！
 * 
 * pbk 2015年10月16日
 */
var LoadMgr = LoadMgr || {
	/** 初始化变量*/
	Init:function(){
		var self = this;
		self._iTypeImage = 1;    // 图片资源
		self._iTypeBinary = 2;   // 二进制资源
		self._iTypeAnimation = 3;// 动画资源
		self._iTypeTexture = 4;  // 纹理资源
		self._iTypeSound = 5;    // 音效资源
		self._iTypeBulk = 6;     // 批量加载资源
		self._iTypeSkeleton = 7;     // skeleton

		// 加载单元缓存对象池
		self._oCachePool = new MLObjectPool( 30, LoadCell );
		
		// 初始化资源缓存列表
		self._resCache = {};
		// 当前正在加载资源列表
		self._loadingList = [];
		// 加载错误限制，三次加载失败后的资源不再加载
		self._loadErrLimit = {};
		
		self._iCurLoadCount = 0;// 当前加载数量
		self._iMaxLoadCount = 16;// 最大同时加载数量

       //TODO start jaye 调整加载限制
        self._iMaxLoadCount = 0xFFFFFFFF;// 最大同时加载数量
        self._tLoadCount = 0;
        self._tMaxLoadCount = 16;
        self._tCDNLoadArr = [];
        //TODO end jaye 调整加载限制

		self._priorList = new Array(); // 优先加载列队
		self._commonList = new Array();// 普通加载列队
		self._preList = new Array();   // 预加载列队

		self._version = new HashMap();
		// ------------------------------------------- cocos2d-js引擎扩展
		//self._ExAudioLoader( cc.__audioSupport );
		self._ExAudioLoader01();
		self._ExBulkLoader();
		
		self.StartAutoRelease();

		zz.Log( "LoadMgr Init" );
		
		return self;
	},
	
	// ====================================================================================================
	// ====================================================================================== 加载图片
	// ====================================================================================================
	/**
	 * 加载单张图片资源
	 * @param url 加载资源路径
	 * @param target 加载对象
	 * @param priority 加载优先级（LoadPriority_Prior最先加载，LoadPriority_Common普通加载，LoadPriority_Preload预加载）
	 * @param args 其他参数{complete:加载完成回调函数（返回类型Image）,completeParam:加载完成回调参数,
	 *                     error:加载是失败回调函数,errorParam:加载失败回调参数,
	 *                     cache:是否缓存资源（默认ture）}
	 */
	LoadImage:function( url, target, priority, args ){
		var self = this;
		
		var cell = self._oCachePool.Pop();
		cell.iType = self._iTypeImage;
		cell.sUrl = url;
		cell.oTarget = target;
		cell.iPriority = priority;
		cell.args = args;

		var img = self._GetCacheRes( cell.sUrl, target );
		if ( img ){
			// 请求时检测缓存资源
			cell.Complete( img );
			self._oCachePool.Push( cell );
		} else {
			self._PushLoadList( cell );
			
			if ( self._iCurLoadCount < self._iMaxLoadCount ){
				self._iCurLoadCount += 1;
				self._LoadNext();
			}
		}
		
		return cell;
	},
	
	_LoadImage:function( cell ){
		var self = this;
		
		var img = self._GetCacheRes( cell.sUrl, cell.oTarget );
		if ( img ){
			// 在开始加载时检测缓存资源
			cell.Complete( img );
			self._oCachePool.Push( cell );
			self._LoadNext();
		} else {
			if ( self._CheckLoadLimit( cell.sUrl ) ){
				cell.Error();
				cc.error( "Error LoadImage Limit! URL:" + cell.sUrl );
				
				self._oCachePool.Push( cell );
				self._LoadNext();
			} else {
				// 加载资源
				self._loadingList.push( cell );
				//zz.Log( "cell.sUrl:" + cell.sUrl );
				self.LoadCdnFile( cell.sUrl, function(){
					cc.loader.loadImg( cell.sUrl, function( err, img ){
						if ( err != null ){
							// 加载错误
							self._AddLoadLimit( cell.sUrl );
							self.ReleaseCacheRes( cell.sUrl, cell.oTarget );
							cell.Error();
							cc.error( "Error LoadImage! URL:" + cell.sUrl + " info:" + err );
						} else {
							// 加载成功
							if ( cell.IsCache ){
								self._AddCacheRes( cell.sUrl, cell.oTarget, img, cell.iType );
							}

							cell.Complete( img );
						}

						DelArrayElement( self._loadingList, cell );
						self._oCachePool.Push( cell );
						self._LoadNext();
					});
				} );
			}
		}
	},
	
	// ====================================================================================================
	// ====================================================================================== 加载二进制文件
	// ====================================================================================================
	/**
	 * 加载单个二进制文件
	 * @param url 加载资源路径
	 * @param target 加载对象
	 * @param priority 加载优先级（LoadPriority_Prior最先加载，LoadPriority_Common普通加载，LoadPriority_Preload预加载）
	 * @param args 其他参数{complete:加载完成回调函数（返回类型Uint8Array）,completeParam:加载完成回调参数,
	 *                     error:加载是失败回调函数, errorParam:加载失败回调参数,
	 *                     cache:是否缓存资源（默认ture）}
	 */
	LoadBinary:function( url, target, priority, args ){
		var self = this;

		var cell = self._oCachePool.Pop();
		cell.iType = self._iTypeBinary;
		cell.sUrl = url;
		cell.oTarget = target;
		cell.iPriority = priority;
		cell.args = args;

		var datas = self._GetCacheRes( cell.sUrl, target );
		if ( datas ){
			// 请求时检测缓存资源
			cell.Complete( datas );
			self._oCachePool.Push( cell );
		} else {
			self._PushLoadList( cell );
			
			if ( self._iCurLoadCount < self._iMaxLoadCount ){
				self._iCurLoadCount += 1;
				self._LoadNext();
			}
		}
		
		return cell;
	},
	
	_LoadBinary:function( cell ){
		var self = this;
		var datas = self._GetCacheRes( cell.sUrl, cell.oTarget );
		if ( datas ){
			// 在开始加载时检测缓存资源
			cell.Complete( datas );
			self._oCachePool.Push( cell );
			self._LoadNext();
		} else {
			if ( self._CheckLoadLimit( cell.sUrl ) ){
				cell.Error();
				cc.error( "Error LoadBinary Limit! URL:" + cell.sUrl );

				self._oCachePool.Push( cell );
				self._LoadNext();
			} else {
				self._loadingList.push( cell );

				self.LoadCdnFile( cell.sUrl, function() {
					if( jsb.fileUtils.isFileExist(cell.sUrl)){
						cc.loader.loadBinary(cell.sUrl, function (err, datas) {
							if (err != null) {
								// 加载错误
								self._AddLoadLimit(cell.sUrl);
								self.ReleaseCacheRes(cell.sUrl, cell.oTarget);
								cell.Error();
								cc.error("Error LoadBinary! URL:" + cell.sUrl + " info:" + err);
							} else {
								// 加载成功
								if (cell.IsCache) {
									self._AddCacheRes(cell.sUrl, cell.oTarget, datas, cell.iType);
								}

								cell.Complete(datas);
							}

							DelArrayElement(self._loadingList, cell);
							self._oCachePool.Push(cell);
							self._LoadNext();
						});
					}
					else{
						self._AddLoadLimit(cell.sUrl);
						self.ReleaseCacheRes(cell.sUrl, cell.oTarget);
						cell.Error();
						cc.error("Error LoadBinary! URL:" + cell.sUrl );
					}
				});
			}
		}
	},
	
	// ====================================================================================================
	// ====================================================================================== 加载动画
	// ====================================================================================================
	/**
	 * 加载单套动画资源
	 * @param url 加载资源路径，url参数传入不要带后缀名，而且资源配置文字plist必须和png图像文件同名
	 * @param target 加载对象
	 * @param priority 加载优先级（LoadPriority_Prior最先加载，LoadPriority_Common普通加载，LoadPriority_Preload预加载）
	 * @param args 其他参数{complete:加载完成回调函数（返回类型Object）,completeParam:加载完成回调参数,
	 *                     error:加载是失败回调函数,errorParam:加载失败回调参数,
	 *                     cache:是否缓存资源（默认ture）}
	 */
	LoadAnimation:function( url, target, priority, args ){
		var self = this;
		
		var cell = self._oCachePool.Pop();
		cell.iType = self._iTypeAnimation;
		cell.sUrl = url;
		cell.oTarget = target;
		cell.iPriority = priority;
		cell.args = args;

		var obj = self._GetCacheRes( cell.sUrl, target );
		if ( obj ){
			// 请求时检测缓存资源
			cell.Complete( obj );
			self._oCachePool.Push( cell );
		} else {
			self._PushLoadList( cell );
			
			if ( self._iCurLoadCount < self._iMaxLoadCount ){
				self._iCurLoadCount += 1;
				self._LoadNext();
			}
		}
		
		return cell;
	},

	animCfgMap:{},	//json解析较耗时间 这个就不释放了
	_LoadAnimation:function( cell ){
		var self = this;

		var obj = self._GetCacheRes( cell.sUrl, cell.oTarget );
		if ( obj ){
			// 在开始加载时检测缓存资源
			cell.Complete( obj );
			self._oCachePool.Push( cell );
			self._LoadNext();
		} else {
			if ( self._CheckLoadLimit( cell.sUrl ) ){
				cell.Error();
				cc.error( "Error LoadAnimationTxt Limit! URL:" + cell.sUrl );

				self._oCachePool.Push( cell );
				self._LoadNext();
			} else {
				self._loadingList.push( cell );

				var loadAnimationImg = function(){
					self.LoadCdnFile( cell.sUrl + ".png", function() {
						cc.loader.loadImg(cell.sUrl + ".png", function (err, img) {
							if (err != null) {
								// 加载错误
								self._AddLoadLimit(cell.sUrl);
								self.ReleaseCacheRes(cell.sUrl, cell.oTarget);
								cell.Error();
								cc.error("Error LoadAnimationImg! URL:" + cell.sUrl + " info:" + err);
							} else {
								var texture2d = null;

								if (EngineCost.engieType == EngineCost.TYPE_WEB) {
									texture2d = new cc.Texture2D();
									texture2d.initWithElement(img);
									texture2d.handleLoadedTexture();
								}
								else
									texture2d = img;
								Profiler.LogFight("SkillEffect.loadAnimationImg");
								var retFrames = self._ParseAnimi(animjson, texture2d);
								var obj = {frames: retFrames, animjson: animjson};
								Profiler.LogFight("SkillEffect.loadAnimationImg", true);

								// 加载成功
								if (cell.IsCache()) {
									self._AddCacheRes(cell.sUrl, cell.oTarget, obj, cell.iType);
								}

								cell.Complete(obj);
							}

							DelArrayElement(self._loadingList, cell);
							self._oCachePool.Push(cell);
							self._LoadNext();
						});
					});
				}

				var animjson = self.animCfgMap[cell.sUrl];
				if( !animjson ){
					self.LoadCdnFile( cell.sUrl + ".json", function() {
						cc.loader.loadTxt(cell.sUrl + ".json", function (err, txt) {
							if (err != null|| !txt) {	//找不到文件jsb不会报错 只会返回空字符串txt
								// 加载错误
								self._AddLoadLimit(cell.sUrl);
								self.ReleaseCacheRes(cell.sUrl, cell.oTarget);
								cell.Error();
								cc.error("Error LoadAnimationTxt! URL:" + cell.sUrl + " info:" + err);

								DelArrayElement(self._loadingList, cell);
								self._oCachePool.Push(cell);
								self._LoadNext();
							} else {
								/* 加载得到的数据先转换成XML格式，再转换成json object格式
								 最后转换成{_inited: true, frames: frames, meta: meta}
								 frames是每帧图片的参数，meta.image有个参数 */
								animjson = JSON.parse(txt);//cc.plistParser.parse(txt);
								//						ClassUtil.printObj(test["frames"]);
								//						var test = cc.spriteFrameCache;
								//						ClassUtil.printObj( test  );
								//						var animjson = PlistParser.parse( txt );
								//						var animjson = cc.PlistParser.parse();

								// animCfg = PlistParser.parseFrameConfig(animjson);
								self.animCfgMap[cell.sUrl] = animjson;

								loadAnimationImg.call(null);

							}
						});
					});
				}
				else{
					loadAnimationImg.call(null);
				}
			}
		}
	},
	
	// 解析生成动画帧文件spriteFrame
	_ParseAnimi:function( animjson, texture ){
		var frames = animjson.frames;// 每帧图片的参数
		var retFrames = [];

		var frame = null;
		var spriteFrame = null;
		var locTexture = null;
		var tempElement = null;
		var tempTexture = null;
		var rect = null;
		
		for ( var i = 0, len = frames.length; i < len; i++ ) {
			frame = frames[i];
			spriteFrame = retFrames[frame.name];

			if ( !spriteFrame ) {
				// 从纹理图片上剪切对应的帧图
				spriteFrame = new cc.SpriteFrame( texture, frame.rect, frame.rotated, frame.offset, animjson.sourceSize );
				
				if ( false/*cc._renderType === cc._RENDER_iType_CANVAS && spriteFrame.isRotated()*/ ) {
					// clip to canvas 显示类型是canvas，且要旋转
					locTexture = spriteFrame.getTexture();
					if (locTexture.isLoaded()) {
						tempElement = spriteFrame.getTexture().getHtmlElementObj();
						tempElement = cc.Sprite.CanvasRenderCmd._cutRotateImageToCanvas(tempElement, spriteFrame.getRectInPixels());

						tempTexture = new cc.Texture2D();
						tempTexture.initWithElement(tempElement);
						tempTexture.handleLoadedTexture();
						spriteFrame.setTexture(tempTexture);

						rect = spriteFrame._rect;
						spriteFrame.setRect(cc.rect(0, 0, rect.width, rect.height));
					}
				}
				// spriteFrame.name = frame.name;
				// cc.spriteFrameCache.addSpriteFrame( spriteFrame, frame.name );
				spriteFrame.retain();	//动画不放spriteFrameCache里  测下removeunusedSpriteFrame
				if(spriteFrame.getTexture() instanceof Object)
					retFrames.push(spriteFrame);
			}
		}
		return retFrames;
	},
	
	// ====================================================================================================
	// ====================================================================================== 加载纹理
	// ====================================================================================================
	/**
	 * 加载单套纹理资源
	 * 主要为加载分块式的纹理资源
	 * 加载的文件格式标准:必须是一个纹理资源图(*.png 或其他)，一个描述文件(*.plist)(如:res/a.plist, res/a.png)
	 * @param url 加载资源路径，url参数传入不要带后缀名，而且资源配置文字plist必须和png图像文件同名
	 * @param target 加载对象
	 * @param priority 加载优先级（LoadPriority_Prior最先加载，LoadPriority_Common普通加载，LoadPriority_Preload预加载）
	 * @param args 其他参数{complete:加载完成回调函数（返回类型Object）,completeParam:加载完成回调参数,
	 *                     error:加载是失败回调函数,errorParam:加载失败回调参数,
	 *                     cache:是否缓存资源（默认ture）}
	 */
	LoadTexture:function( url, target, priority, args ){
		var self = this;
		var cell = self._oCachePool.Pop();
		cell.iType = self._iTypeTexture;
		cell.sUrl = url;
		cell.oTarget = target;
		cell.iPriority = priority;
		cell.args = args;

		var obj = self._GetCacheRes( cell.sUrl, target );
		if ( obj ){
			// 请求时检测缓存资源
			cell.Complete( obj );
			self._oCachePool.Push( cell );
		} else {
			self._PushLoadList( cell );
			
			if ( self._iCurLoadCount < self._iMaxLoadCount ){
				self._iCurLoadCount += 1;
				self._LoadNext();
			}
		}

		return cell;
	},
	
	_LoadTexture:function( cell ){
		var self = this;

//		zz.Log( "cell.sUrl:" + cell.sUrl );
		var obj = self._GetCacheRes( cell.sUrl, cell.oTarget );
		if ( obj ){
			// 在开始加载时检测缓存资源
			cell.Complete( obj );
			self._oCachePool.Push( cell );
			self._LoadNext();
		} else {
			if ( self._CheckLoadLimit( cell.sUrl ) ){
				cell.Error();
				cc.error( "Error LoadTextureTxt Limit! URL:" + cell.sUrl );

				self._oCachePool.Push( cell );
				self._LoadNext();
			} else {
				self._loadingList.push( cell );

				self.LoadCdnFile( cell.sUrl + ".json", function() {
					cc.loader.loadTxt(cell.sUrl + ".json", function (err, txt) {
                        if (err != null|| !txt) {	//找不到文件jsb不会报错 只会返回空字符串txt
							// 加载错误
							self._AddLoadLimit(cell.sUrl);
							self.ReleaseCacheRes(cell.sUrl, cell.oTarget);
							cell.Error();
							cc.error("Error LoadTextureTxt! URL:" + cell.sUrl + " info:" + err);

							DelArrayElement(self._loadingList, cell);
							self._oCachePool.Push(cell);
							self._LoadNext();
						} else {
							//zz.Log( cell.sUrl + txt );
							var textureJson = JSON.parse(txt);//PlistParser.parseFrameConfig(cc.plistParser.parse(txt));

							self.LoadCdnFile( cell.sUrl + ".png", function() {
//														zz.Log( "cell.sUrl:" + cell.sUrl + ".png" );
								cc.loader.loadImg(cell.sUrl + ".png", function (err, img) {
									//							zz.Log( "downing cell.sUrl:" + cell.sUrl + ".png" );
									if (err != null) {
										// 加载错误
										self._AddLoadLimit(cell.sUrl);
										self.ReleaseCacheRes(cell.sUrl, cell.oTarget);
										cell.Error();
										cc.error("Error LoadTextureImg! URL:" + cell.sUrl + " info:" + err);
									} else {
										var texture2d;
										if (EngineCost.engieType == EngineCost.TYPE_WEB) {
											texture2d = new cc.Texture2D();
											texture2d.initWithElement(img);
											texture2d.handleLoadedTexture();
										}
										else
											texture2d = img;

										var retFrames = self._ParseTexture(textureJson, texture2d, cell.args.keyName);

										// 加载成功
										if (cell.IsCache()) {
											self._AddCacheRes(cell.sUrl, cell.oTarget, retFrames, cell.iType);
										}

										cell.Complete(retFrames);
									}

									DelArrayElement(self._loadingList, cell);
									self._oCachePool.Push(cell);
									self._LoadNext();
								});
							});
						}
					});
				});
			}
		}
	},
	
	// 解析生成动画帧文件spriteFrame
	_ParseTexture:function( cfg, texture, panelName ){
		var frames = cfg.frames;// 每帧图片的参数
		var retFrames = {};

		var frame = null;
		var spriteFrame = null;
		var locTexture = null;
		var tempElement = null;
		var tempTexture = null;
		var rect = null;
		for ( var i = 0, len = frames.length; i < len; i++ ) {
			frame = frames[i];
			spriteFrame = retFrames[frame.name];

			if ( !spriteFrame ) {
				// 从纹理图片上剪切对应的帧图
				spriteFrame = new cc.SpriteFrame( texture, frame.rect, frame.rotated, frame.offset, frame.sourceSize );

				if ( cc._renderType === cc._RENDER_iType_CANVAS && spriteFrame.isRotated() ) {
					// clip to canvas 显示类型是canvas，且要旋转
					locTexture = spriteFrame.getTexture();

					if ( false ) {
						tempElement = spriteFrame.getTexture().getHtmlElementObj();
						tempElement = cc.Sprite.CanvasRenderCmd._cutRotateImageToCanvas( tempElement, spriteFrame.getRectInPixels() );

						tempTexture = new cc.Texture2D();
						tempTexture.initWithElement( tempElement );
						tempTexture.handleLoadedTexture();
						spriteFrame.setTexture( tempTexture );
						rect = spriteFrame._rect;
						spriteFrame.setRect( cc.rect( 0, 0, rect.width, rect.height ) );

					}
				}
				retFrames[frame.name] = spriteFrame;
				var name = panelName + "/" + frame.name;
				spriteFrame.name = name;
				cc.spriteFrameCache.addSpriteFrame( spriteFrame, name );	//只是为了兼容ui组件使用plist_texture从spriteFrameCache里取
				spriteFrame.retain();	//spriteFrameCache在内存过高会自动释放	这里才是真正的引用持有 		--merlin
			}
		}

		return retFrames;
	},
	
	// ====================================================================================================
	// ====================================================================================== 加载音效
	// ====================================================================================================
	/**
	 * 加载单个音效资源，支持格式"mp3", "ogg", "wav", "mp4", "m4a"
	 * 加载的资源会自动缓存
	 * @param url 加载资源路径
	 * @param target 加载对象
	 * @param priority 加载优先级（LoadPriority_Prior最先加载，LoadPriority_Common普通加载，LoadPriority_Preload预加载）
	 * @param args 其他参数{complete:加载完成回调函数（返回类型AudioEx）,completeParam:加载完成回调参数,
	 *                     error:加载是失败回调函数,errorParam:加载失败回调参数,
	 *                     cache:是否缓存资源（默认ture）}
	 */
	LoadSound:function( url, target, priority, args ){
		var self = this;

		var cell = self._oCachePool.Pop();
		cell.iType = self._iTypeSound;
		cell.sUrl = url;
		cell.oTarget = target;
		cell.iPriority = priority;
		cell.args = args;

		var audio = self._GetCacheRes( cell.sUrl, target );
		if ( audio ){
			// 请求时检测缓存资源
			if(audio.IsPlaying()>0) {
                audio = audio.CloneNode();
            }
			cell.Complete( audio );
			self._oCachePool.Push( cell );
		} else {
			self._PushLoadList( cell );
			
			if ( self._iCurLoadCount < self._iMaxLoadCount ){
				self._iCurLoadCount += 1;
				self._LoadNext();
			}
		}
		
		return cell;
	},
	
	_LoadSound:function( cell ){
		var self = this;
		var audio = self._GetCacheRes( cell.sUrl, cell.oTarget );
		if ( audio ){
			// 在开始加载时检测缓存资源
			cell.Complete( audio );
			self._oCachePool.Push( cell );
			self._LoadNext();
		} else {
			if ( self._CheckLoadLimit( cell.sUrl ) ){
				cell.Error();
				cc.error( "Error LoadSound Limit! URL:" + cell.sUrl );

				self._oCachePool.Push( cell );
				self._LoadNext();
			} else {
				self._loadingList.push( cell );

				self.LoadCdnFile( cell.sUrl, function() {
					self.audioLoader.load(cell.sUrl, null, null, function (err, audio) {
						if (err != null) {
							// 加载错误
							self._AddLoadLimit(cell.sUrl);
							self.ReleaseCacheRes(cell.sUrl, cell.oTarget);
							cell.Error();
							cc.error("Error LoadSound! URL:" + cell.sUrl + " info:" + err);
						} else {
							// 加载成功
							if (cell.IsCache()) {
								self._AddCacheRes(cell.sUrl, cell.oTarget, audio, cell.iType);
							}

							cell.Complete(audio);
						}

						DelArrayElement(self._loadingList, cell);
						self._oCachePool.Push(cell);
						self._LoadNext();
					}, cell.args);
				});
			}
		}
	},
	
	// ====================================================================================================
	// ====================================================================================== 批量加载资源
	// ====================================================================================================
	/**
	 * 批量加载资源（不支持动画和纹理资源，不会进行同时加载重复检测）
	 * @param urls 加载资源路径列表
	 * @param target 加载对象
	 * @param priority 加载优先级（LoadPriority_Prior最先加载，LoadPriority_Common普通加载，LoadPriority_Preload预加载）
	 * @param args 其他参数{complete:加载完成回调函数（返回类型：Array，加载失败对应位置数据为null）,completeParam:加载完成回调参数,
	 *                     error:加载是失败回调函数,errorParam:加载失败回调参数,
	 *                     progress:加载中回调函数（返回：加载数据，对应下标，总加载个数，加载完成个数，自定义参数）,progressParam:加载中回调参数,
	 *                     cache:是否缓存资源（默认ture）}
	 */
	LoadBulk:function( urls, target, priority, args ){
		var self = this;

		if ( !(urls instanceof Array) ){
			urls = [urls];
		}
		
		var cell = self._oCachePool.Pop();
		cell.iType = self._iTypeBulk;
		cell.sUrl = urls;
		cell.sCdnUrl = urls;
		cell.oTarget = target;
		cell.iPriority = priority;
		cell.args = args;

		// 请求时检测缓存资源
		if ( self._HasCacheAllRes( cell.sUrl ) ){
			var retList = new Array();
			var len = cell.sUrl.length;
			for ( var i = 0; i < len; ++i ){
				retList.push( self._GetCacheRes( cell.sUrl[i], target ) );
			}
			
			cell.Complete( retList );
			self._oCachePool.Push( cell );
		} else {
			self._PushLoadList( cell );
			
			if ( self._iCurLoadCount < self._iMaxLoadCount ){
				self._iCurLoadCount += 1;
				self._LoadNext();
			}
		}

		return cell;
	},
	
	_LoadBulk:function( cell ){
		var self = this;

		var asyncPool = new cc.AsyncPool( 
			cell.sUrl, 0, 
			function( url, index, asyncPoolCallback, aPool ){// 单项资源开始加载资源回调
				var data = self._GetCacheRes( url, cell.oTarget );
				if ( data ){
					// 读取缓存资源
					cell.Progress( data, index, aPool.size, aPool.finishedSize + 1 );
					asyncPoolCallback( null, data );
				} else {
					if ( self._CheckLoadLimit( cell.sUrl ) ){
						// 加载限制
						cell.Progress( null, index, aPool.size, aPool.finishedSize + 1 );
						cc.error( "Error LoadBulk Limit! Index:" + index + " URL:" + url );
						asyncPoolCallback( "Load limit!", null );
					} else {
						// 加载单项资源
						self._LoadResIterator( url, index, 
							function( err, data ){// 单项资源加载完成回调
								if ( err != null ){
									// 加载错误
									self._AddLoadLimit( url );
									self.ReleaseCacheRes( url, cell.oTarget );
									cell.Error();
								} else {
									// 加载成功
									if ( cell.IsCache() ){
										self._AddCacheRes( url, cell.oTarget, data, cell.iType);
									}
								}

								cell.Progress( data, index, aPool.size, aPool.finishedSize + 1 );
								asyncPoolCallback( err, data );
							},
							cell.args
						);
					}
				}
			},
			function( err, retList ){// 列表所有资源加载完成后回调
				// 所有加载成功
				cell.Complete( retList );

				self._oCachePool.Push( cell );
				self._LoadNext();
			},
			null
		);
		
		asyncPool.flow();
	},

    // ====================================================================================================
    // ====================================================================================== 加载骨骼
    // ====================================================================================================
    /**
     * 加载骨骼资源
     * @param url 加载资源路径，url参数传入不要带后缀名，而且资源配置文字atalas json png同名
     * @param target 加载对象
     * @param priority 加载优先级（LoadPriority_Prior最先加载，LoadPriority_Common普通加载，LoadPriority_Preload预加载）
     * @param args 其他参数{complete:加载完成回调函数（返回类型Object）,completeParam:加载完成回调参数,
     *                     error:加载是失败回调函数,errorParam:加载失败回调参数,
     *                     cache:是否缓存资源（默认ture）}
     */
    LoadSkeletonAnimation:function( url, target, priority, args ){
        var self = this;

        var cell = self._oCachePool.Pop();
        cell.iType = self._iTypeSkeleton;
        cell.sUrl = url;
        cell.oTarget = target;
        cell.iPriority = priority;
        cell.args = args;

        var obj = self._GetCacheRes( cell.sUrl, target );
        if ( obj ){
            // 请求时检测缓存资源
            cell.Complete( obj );
            self._oCachePool.Push( cell );
        } else {
            self._PushLoadList( cell );

            if ( self._iCurLoadCount < self._iMaxLoadCount ){
                self._iCurLoadCount += 1;
                self._LoadNext();
            }
        }

        return cell;
    },
    /// 这货没有导出JS接口，
    _LoadSkeletonAnimation:function( cell ){
        var self = this;
        var obj = self._GetCacheRes( cell.sUrl, cell.oTarget );
        if ( obj ){
            // 在开始加载时检测缓存资源
            cell.Complete( obj );
            self._oCachePool.Push( cell );
            self._LoadNext();
        } else {
            if ( self._CheckLoadLimit( cell.sUrl ) ){
                cell.Error();
                cc.error( "Error LoadAnimationTxt Limit! URL:" + cell.sUrl );

                self._oCachePool.Push( cell );
                self._LoadNext();
            } else {
                self._loadingList.push( cell );

                var fileArr = [
                    ".atlas",
                    ".json",
                    ".png",
                ]
                var index = 0
                var loadAnimationImg = function(){
                    var file = fileArr[index++]
                    self.LoadCdnFile( cell.sUrl + file, function() {
                        if( jsb.fileUtils.isFileExist(cell.sUrl + file)){
                            if(index != fileArr.length){
                                loadAnimationImg()
                                return
                            }

                            let datas = {
                                atlas : cell.sUrl + fileArr[0],
                                json : cell.sUrl + fileArr[1],
                                png : cell.sUrl + fileArr[2],
                            }
                            // 加载成功
                            if (cell.IsCache) {
                                self._AddCacheRes(cell.sUrl, cell.oTarget, datas, cell.iType);
                            }

                            cell.Complete(datas);

                            DelArrayElement(self._loadingList, cell);
                            self._oCachePool.Push(cell);
                            self._LoadNext();
                        }
                        else{
                            self._AddLoadLimit(cell.sUrl);
                            self.ReleaseCacheRes(cell.sUrl, cell.oTarget);
                            cell.Error();
                            cc.error("Error Skeleton! URL:" + cell.sUrl );
                        }
                    });
                }
                loadAnimationImg()
            }
        }
    },

	// ====================================================================================================
	// ====================================================================================== 循环加载
	// ====================================================================================================
	// 加载下一个资源，在上一个资源加载完成后调用
	_LoadNext:function(){
		var self = this;
		
		var cell = self._PopLoadList();
		if ( cell == null ){
			self._iCurLoadCount -= 1;
			return;
		}

		if ( cell.TargetIsDispose() ){
			self._oCachePool.Push( cell );
			self._LoadNext();
			return;
		}
		
		if ( cell.iType === self._iTypeImage ){
			self._LoadImage( cell );
		} else if ( cell.iType === self._iTypeBinary ){
			self._LoadBinary( cell );
		} else if ( cell.iType === self._iTypeAnimation ){
			self._LoadAnimation( cell );
		} else if ( cell.iType === self._iTypeTexture ){
			self._LoadTexture( cell );
		} else if ( cell.iType === self._iTypeSound ){
			self._LoadSound( cell );
		} else if ( cell.iType === self._iTypeBulk ){
			self._LoadBulk( cell );
        } else if ( cell.iType === self._iTypeSkeleton ){
            self._LoadSkeletonAnimation( cell );
		} else {
			self._LoadNext();
			return;
		}
	},
	
	// ====================================================================================================
	// ====================================================================================== 加载限制
	// ====================================================================================================
	// 添加加载限制
	_AddLoadLimit:function( url ){
		var self = this;
		
		if ( !self._loadErrLimit[url] ){
			self._loadErrLimit[url] = 1;
		} else {
			self._loadErrLimit[url] += 1;
		}
	},
	
	// 检测加载限制
	_CheckLoadLimit:function( url ){
		return this._loadErrLimit[url] && this._loadErrLimit[url] > 3;
	},
	
	// ====================================================================================================
	// ====================================================================================== 资源缓存
	// ====================================================================================================
	/**
	 * 开启自动释放缓存
	 */
	StartAutoRelease:function(){
		if ( this.OpenAuto )
			return;

		this.OpenAuto = TimerUtil.TimeUpdate( this._AutoReleaseLoc.bind( this ), this, 5 );// 每5秒检测一次
	},
	
	/**
	 * 关闭自动释放缓存
	 */
	StopAutoRelease:function(){
		if ( !this.OpenAuto )
			return;

		TimerUtil.ClearTimeUpdate( this.OpenAuto, this );
		
		this.OpenAuto = false;
	},
	
	_AutoReleaseLoc:function(){
		var self = this;

		var cache = self._resCache;
		var time = new Date().getTime();
		
		var cacheData = null;
		var idx = null, len = null;
		var target = null;
		
		for ( var url in cache ){
			cacheData = cache[url];
			if ( cacheData.always )
				continue;
			
			len = cacheData.list.length;
			
			for ( idx = len - 1; idx >= 0; --idx ){
				target = cacheData.list[idx];
				
				if ( target.bDispose ){
					if ( time - cacheData.time > 30000 ){// 资源无被使用，30秒钟后删除
						self.ReleaseCacheRes( url, target );
					}
				} else {
					cacheData.time = time;
				}
			}
		}
		// cc.director.purgeCachedData();
		// cc.spriteFrameCache.removeUnusedSpriteFrames();
		cc.textureCache.removeUnusedTextures();
		// var info = cc.textureCache.getCachedTextureInfo();
		// var infos = info.split("\n");
		// for( var i = 0, len = infos.length; i < len; i++ )
		// 	cc.log( infos[i] );
	},
	
	// 添加缓存资源
	_AddCacheRes:function( url, target, data, type ){
		//zz.Log( "AddCacheRes:" + url );
		if ( !this._resCache[url] ){
			if ( !target ){
				this._resCache[url] = { "data":data, "always":true, "list":[], "type":type };
			} else {
				this._resCache[url] = { "data":data, "always":false, "time":new Date().getTime(), "list":[target], "type":type };
			}
			switch (type){
				case this._iTypeImage:
					data.retain();
					break;
				case this._iTypeAnimation:		//动画与图集在加载那里retain的
				case this._iTypeTexture:
					break;
			}
//			if ( typeof data == "SpriteFrame"  )
//				cc.spriteFrameCache.addSpriteFrame(data, url);
			
		} else {
			var cacheData = this._resCache[url]; 
			
			if ( !target ){
				cacheData.always = true;
			} else {
				if ( cacheData.list.indexOf( target ) == -1 ){
					cacheData.list.push( target );
				}
				
				cacheData.time = new Date().getTime();
			}
		}
	},
	
	// 获取缓存资源
	_GetCacheRes:function( url, target ){
		var cacheData = this._resCache[url];
		
		if ( !cacheData ){
			return null;
		} else {
			if ( !target ){
				cacheData.always = true;
			} else {
				if ( cacheData.list.indexOf( target ) == -1 ){
					cacheData.list.push( target );
				}
			}
			
			cacheData.time = new Date().getTime();
			
			return cacheData.data;
		}
	},
	
	// 是否缓存有列表的所有资源
	_HasCacheAllRes:function( urls ){
		var cache = this._resCache;
		var len = urls.length;
		
		for ( var i = 0; i < len; ++i ){
			if ( !cache[urls[i]] )
				return false;
		}
		
		return true;
	},
	
	/**
	 * 释放缓存资源
	 * @param url
	 * @param target
	 * @param force 强制释放 默认是false
	 */
	ReleaseCacheRes:function( url, target, force ){
		var cacheData = this._resCache[url];
		
		if ( !cacheData ){
			return;
		}
		
		if ( arguments.length >= 3 && force ){
			delete this._resCache[url];
			return;
		}
		
		if ( !target ){
			return;
		}
		
		var idx = cacheData.list.indexOf( target );
		if ( idx == -1 )
			return;
		
		cacheData.list.splice( idx, 1 );
		
		if ( cacheData.list.length == 0 && !cacheData.always ){
			switch (cacheData.type){
				case this._iTypeImage:
					cacheData.data.release();
					break;
				case this._iTypeAnimation:
					var len = cacheData.data.frames.length;
					for(var i = 0; i < len; i++ ){
						cacheData.data.frames[i].release();
					}
					break;
				case this._iTypeTexture:
					for( var j in cacheData.data ){
						var frame = cacheData.data[j];
						frame.release();
						cc.spriteFrameCache.removeSpriteFrameByName( frame.name );
					}
					break;
			}
			delete this._resCache[url];
		}
	},
	
	/**
	 * 强制释放所有缓存资源
	 */
	ReleaseAllCacheRes:function(){
		var cache = this._resCache;
		
		for ( var url in cache ){
			delete cache[url];
		}
	},
	
	// ====================================================================================================
	// ====================================================================================== 加载队列
	// ====================================================================================================
	// 在等待列队中放入待加载资源
	_PushLoadList:function( cell, priority ){
		if ( priority === LoadPriority_Common ){
			this._commonList.push( cell );
		} else if ( priority === LoadPriority_Prior ){
			this._priorList.push( cell );
		} else {
			this._preList.push( cell );
		}
	},

	// 取出下一个待加载资源
	_PopLoadList:function(){
		var self = this;
		var cell = null;

		if ( self._priorList.length > 0 ){
			cell = self._GetCanLoadCell( self._priorList );
			if ( cell ){
				return cell;
			}
		}

		if ( self._commonList.length > 0 ){
			cell = self._GetCanLoadCell( self._commonList );
			if ( cell ){
				return cell;
			}
		}
			
		if ( self._preList.length > 0 ){
			cell = self._GetCanLoadCell( self._preList );
			if ( cell ){
				return cell;
			}
		}

		return cell;
	},
	
	// 获取能加载的单元
	_GetCanLoadCell:function( list ){
		var self = this;
		var iList = self._loadingList;
		var iLen = iList.length;
		var cell, i, j;
		
		for ( i = 0; i < list.length; ++i ){
			cell = list[i];
			for ( j = 0; j < iLen; ++j ){
				if ( cell.sUrl == iList[j].sUrl ){
					j = -1;
					break;
				}
			}
			
			if ( j !== -1 ){
				list.splice( i, 1 );
				return cell;
			}
		}
		
		return null;
	},	

	/**
	 * 从加载列表中移除待加载资源
	 * @param cell
	 * @returns 是否移除成功
	 */
	Unload:function( cell ){
		if ( cell == null )
			return false;

		if ( cell.priority === LoadPriority_Common ){
			return DelArrayElement( this._commonList, cell );
		} else if ( cell.priority === LoadPriority_Prior ){
			return DelArrayElement( this._priorList, cell );
		} else {
			return DelArrayElement( this._preList, cell );
		}
	},
	
	// ====================================================================================================
	// ====================================================================================== 加载器状态日志
	// ====================================================================================================
	/** 获取当前加载状态*/
	GetLoadState:function(){
		var self = this;
		
		var logStr = "====== LoadMgr Load State ======"
			+ "\nCurrent Load Count: " + self._iCurLoadCount
			+ "\nPrior List Count: " + self._preList.length
			+ "\nCommon List Count: " + self._commonList.length
			+ "\nPre List Count: " + self._preList.length;
		
		logStr += "\n-----Loading: ";
		var loading = self._loadingList;
		for ( var i = 0; i < loading.length; ++i ){
			logStr += ("\n" + loading[i].sUrl);
		}
		
		logStr += "\n-----Not Load: ";
		var limit = self._loadErrLimit;
		for ( var url in limit ){
			if ( limit[url] > 3 ){
				logStr += ("\n" + url);
			}
		}
		
		logStr += "\n================================";
		
		return logStr;
	},
	
	/** 获取当前缓存状态*/
	GetCacheState:function(){
		var cache = this._resCache;;
		var logStr = "====== LoadMgr Cache State ======";
		
		var cacheData = null;
		for ( var url in cache ){
			cacheData = cache[url];
			
			logStr += ("\n" + url + "  User:" + cacheData.list.length + (cacheData.always ? "  Always" : ""));
		}
		
		logStr += "\n================================";
		
		return logStr;
	},

	// ====================================================================================================
	// ====================================================================================== 扩展：音频加载
	// ====================================================================================================
	_ExAudioLoader01:function()
	{
		var support = [".mp3",".ogg",".wav"];
		this.audioLoader = {
			load:function( realUrl, url, res, cb, args ){
				if ( cc.loader.audioPath )// 添加音效前缀路径
				{
					realUrl = cc.path.join( cc.loader.audioPath, realUrl );
				}
				var extname = cc.path.extname(realUrl);// 获取资源扩展名//.mp3
				var typeList = [extname];
				for ( i = 0; i < support.length; i++ ){
					if ( extname !== support[i] ){
						typeList.push( support[i] );
					}
				}
				// 音频类型
				var soundType = 0;
				if ( args && args.completeParam && args.completeParam.type ){
					soundType = args.completeParam.type;
				}
				var audio  = new AudioEx(null, soundType, realUrl);// url路径加载
				this._loadAudioFromExtList01( realUrl, typeList, audio, cb );
			},
			_loadAudioFromExtList01:function( realUrl, typeList, audio, cb ){
				realUrl = cc.path.changeExtname(realUrl, typeList.splice(0, 1));
                jsb.AudioEngine.preload(realUrl,function(state){
                    cb(null, audio);
                });
                // cb(null, audio);
			}
		};
	},
	// 重写扩展音频加载
	_ExAudioLoader:function( polyfill )
	{
		var SWA = polyfill.webAudio,
			SWB = polyfill.multichannel,
			SWC = polyfill.auto;

		var support = [];

		// 检测支持可播放类型
		var audio = document.createElement("audio");
		if ( audio.canPlayType ) {
			var ogg = audio.canPlayType('audio/ogg; codecs="vorbis"');
			if (ogg && ogg !== "") support.push(".ogg");
			var mp3 = audio.canPlayType("audio/mpeg");
			if (mp3 && mp3 !== "") support.push(".mp3");
			var wav = audio.canPlayType('audio/wav; codecs="1"');
			if (wav && wav !== "") support.push(".wav");
			var mp4 = audio.canPlayType("audio/mp4");
			if (mp4 && mp4 !== "") support.push(".mp4");
			var m4a = audio.canPlayType("audio/x-m4a");
			if (m4a && m4a !== "") support.push(".m4a");
		}
		
		try {
			if ( SWA ){
				var context = new (window.AudioContext || window.webkitAudioContext || window.mozAudioContext)();
				if ( polyfill.delay ){
					setTimeout(
						function(){context = new (window.AudioContext || window.webkitAudioContext || window.mozAudioContext)();}, 
						0);
				}
			}
		} catch( error ) {
			SWA = false;
			zz.Log( "browser don't support webAudio" );
		}
		
		this.audioLoader = {
			load:function( realUrl, url, res, cb, args ){
				if ( support.length === 0 )// 不支持所有的音效
					return cb("can not support audio!");
				
				if ( cc.loader.audioPath )// 添加音效前缀路径
					realUrl = cc.path.join( cc.loader.audioPath, realUrl );
				
				var extname = cc.path.extname(realUrl);// 获取资源扩展名

				var typeList = [extname];
				for ( i = 0; i < support.length; i++ ){
					if ( extname !== support[i] ){
						typeList.push( support[i] );
					}
				}
				
				// 音频类型
				var soundType = 0;
				if ( args && args.completeParam && args.completeParam.type ){
					soundType = args.completeParam.type;
				}
				
				var audio;
				if ( SWA ){
					try {
						audio = new AudioEx( context, soundType, realUrl );// webAudio加载
						if ( polyfill.manualLoop )
							audio._manualLoop = true;
					} catch( err ) {
						SWA = false;
						zz.Log( "browser don't support webAudio" );
						audio = new AudioEx( null, soundType, realUrl );
					}
				}else{
					audio = new AudioEx(null, soundType, realUrl);// url路径加载
				}
				
				this._loadAudioFromExtList( realUrl, typeList, audio, cb );
			},
			
			_loadAudioFromExtList:function( realUrl, typeList, audio, cb ){
				if ( typeList.length === 0 ){
					// 加载资源是不支持的类型
					var ERRSTR = "can not found the resource of audio! Last match url is : ";
					ERRSTR += realUrl.replace(/\.(.*)?$/, "(");
					support.forEach( function(ext){
						ERRSTR += ext + "|";
					});
					ERRSTR = ERRSTR.replace(/\|$/, ")");
					return cb( {status:520, errorMessage:ERRSTR}, null );
				}
				
				realUrl = cc.path.changeExtname(realUrl, typeList.splice(0, 1));

				var self = this;
				
				if ( SWA ){//Buffer
					
					if ( polyfill.webAudioCallback )
						polyfill.webAudioCallback( realUrl );
					
					// ======================== XMLHttpRequest方式加载
					var request = new XMLHttpRequest();
					request.open( "GET", realUrl, true );
					request.responseType = "arraybuffer";

					// Our asynchronous callback
					request.onload = function () {
						context["decodeAudioData"](request.response, function(buffer){
							//success 加载成功
							audio.SetBuffer(buffer);
							cb( null, audio );
						}, function(){
							//error 加载失败，重新加载
							self._loadAudioFromExtList(realUrl, typeList, audio, cb);
						});
					};

					request.onerror = function(){
						// 加载失败，返回错误回调
						cb( {status:520, errorMessage:ERRSTR}, null );
					};

					request.send();
				} else {//DOM
					// ======================== html标签audio，文件方式加载
					var element = document.createElement("audio");
					var cbCheck = false;
					var termination = false;// 超时终止

					var timer = setTimeout(function(){
						if ( element.readyState === 0 ){
							emptied();
						} else {
							termination = true;
							element.pause();
							document.body.removeChild(element);
							cb( "audio load timeout : " + realUrl, audio );
						}
					}, 10000);// 10秒超时

					var success = function(){
						if ( !cbCheck ){
							element.pause();
							try { 
								element.currentTime = 0;
								element.volume = 1; 
							} catch (e) {}
							
							document.body.removeChild(element);
							audio.SetElement(element);
							
							element.removeEventListener("canplaythrough", success, false);
							element.removeEventListener("error", failure, false);
							element.removeEventListener("emptied", emptied, false);
							
							!termination && cb(null, audio);
							cbCheck = true;
							clearTimeout(timer);
						}
					};

					var failure = function(){
						if(!cbCheck) 
							return;
						
						element.pause();
						document.body.removeChild(element);
						
						element.removeEventListener("canplaythrough", success, false);
						element.removeEventListener("error", failure, false);
						element.removeEventListener("emptied", emptied, false);
						
						!termination && self._loadAudioFromExtList( realUrl, typeList, audio, cb );
						cbCheck = true;
						clearTimeout(timer);
					};

					var emptied = function(){
						termination = true;
						success();
						cb(null, audio);
					};

					cc._addEventListener(element, "canplaythrough", success, false);
					cc._addEventListener(element, "error", failure, false);
					if ( polyfill.emptied )
						cc._addEventListener(element, "emptied", emptied, false);

					element.src = realUrl;
					document.body.appendChild( element );
					element.volume = 0;
					element.play();
				}
			},
		};
		
		cc.loader.register( ["mp3", "ogg", "wav", "mp4", "m4a"], this.audioLoader );
	},
	
	// ====================================================================================================
	// ====================================================================================== 扩展：批量加载
	// ====================================================================================================
	// 重写扩展批量加载
	_ExBulkLoader:function(){
		// ------------------------------------- 批量加载添加res格式二进制加载方式
		cc._binLoader = {
				load:function( realUrl, url, res, cb ){
					cc.loader.loadBinary( realUrl, cb );
				}
		};

		cc.loader.register( ["res"], cc._binLoader );
	},
	
	/*
	 * 迭代加载资源器
	 * @param url
	 * @param index
	 * @param cb
	 * @param args
	 */
	_LoadResIterator:function( item, index, cb, args ){
		var self = this,
			ccLoader = cc.loader, 
			url = null;
		
		var type = item.type;
		if ( type ) {
			type = "." + type.toLowerCase();
			url = item.src ? item.src : item.name + type;
		} else {
			url = item;
			type = cc.path.extname( url );
		}

		var loader = null;
		if ( type ) {
			loader = ccLoader._register[type.toLowerCase()];
		}
		
		if ( !loader ) {
			zz.Error( "loader for [" + type + "] not exists!" );
			return cb();
		}
		
		var realUrl = url;
		if ( !UIUtil.urlRegExp().test(url) )
		{
			var basePath = loader.getBasePath ? loader.getBasePath() : ccLoader.resPath;
			realUrl = ccLoader.getUrl( basePath, url );
		}

		if( cc.game.config["noCache"] && typeof realUrl === "string" ){
			if( ccLoader._noCacheRex.test(realUrl) )
				realUrl += "&_t=" + (new Date() - 0);
			else
				realUrl += "?_t=" + (new Date() - 0);
		}

		this.LoadCdnFile( realUrl, function(){
			loader.load(realUrl, url, item, function (err, data) {
				if ( err ) {
					zz.Error( "Error _LoadResIterator! Index:" + index + " URL:" + url + " info:" + err );
					cb( {status:520, errorMessage:err}, null );
				} else {
					cb( null, data );
				}
			}, args);
		});
	},

	InitVersion:function(){
		var self = this;
		cc.loader.loadBinary("updatelisthash.res", function (err, data) {
			if (err != null) {
				zz.Error("Version Map md5 Error: " + err);
			} else {
				var parseData = new Zlib.Inflate( data ).decompress();
				var stream = new BinStream( parseData );
				stream.endian = BinStream.LITTLE_ENDIAN;

				var url;
				url = stream.Utf8ArrayToStr( stream.ReadBytes( stream.bytesAvailable() ) );
				var obj = JSON.parse(url);
				var parseVersion = function( pre, obj){
					for( var key in obj ){
						if( obj[key] instanceof Object) {
							parseVersion(pre + key + "/", obj[key]);
						}
						else{
							self._version.Put( pre + key, obj[key] );
						}
					}
				}
				parseVersion("", obj);
			}
		});
		jsb.fileUtils.addSearchPath( jsb.fileUtils.getWritablePath() + GameConfig.downloadPath, true );
		cc.log("InitVersion SearchPath --> " + JSON.stringify(jsb.fileUtils.getSearchPaths()));
	},


	_downloadCallback:{},
	/**
	 * 如果文件不存在 或者crc32校验不是最新版本 则从网络下载并保存到本地
	 * @param url
	 * @param cb
	 * @constructor
	 */
	LoadCdnFile:function( url, cb ){
		if( !GameConfig.isWebMode ){	//非微端模式 直接返回读本地
			cb();
			return; 
		}
		if( jsb.fileUtils.isFileExist(url)){
			var v = this._version.Get(url);
			if( !v ){		//无远程版本直接返回 像ui这些直接要用json初始化
				cb();
				return;
			}
			TimeTaskMgr.AddTask( TimeTaskMgr.SINGLE_PRIORITY, function(){
				// var md5 = cc.CRC32.GetCRC32(url);//cc.MD5.GetMD5(url);
				var md5 = jsb.fileUtils.getFileSize(url);	//不需要那么严格 直接比size好了 不然性能影响太大
				if( md5 == parseInt(v) ){		//远程版本相同 直接返回	( FixMe 这里如果下载出错中断后断点续传size会下载到服务器的size为止, 实际上内容是不同的 (现在错误改成直接删除了)  -- merlin 2018/06/28)
					cb();
					return;
				}
				this._LoadCdnFile(url, cb, true);
			}, null, this);
			return;
		}
		this._LoadCdnFile(url, cb);
	},

	_LoadCdnFile:function(url, cb, hasOldFile){
		var cdnUrl = GameConfig.WEB_CDN_URL + url;
		var writePath = jsb.fileUtils.getWritablePath() + GameConfig.downloadPath + url;

		var cbArr = this._downloadCallback[writePath];
		if(cbArr){		//之前基于本地同步的加载好像有点问题 现在同时下载同一url估计都会走到这里 先简单做个队列
			cbArr.push(cb);
			cc.error("***********************************************************");
			cc.error("****************     dumplicate url    ********************");
			cc.error(writePath);
			cc.error("***********************************************************");
			return;
		}
		else{
			// //TODO start jaye 调整加载限制
			// if(this._tLoadCount < this._tMaxLoadCount){
			// 	this._tLoadCount++;
			// }else{
			// 	this._tCDNLoadArr.push({
			// 		url : url,
			// 		cb : cb,
			// 	})
			// 	return;
			// }
			//TODO end jaye 调整加载限制
			this._downloadCallback[writePath] = [cb];
		}
		var self = this;
		if( !this._downloader ){
			this._downloader = new jsb.Downloader({"countOfMaxProcessingTasks":8, "timeoutInSeconds":20, "tempFileNameSuffix":".tmp" });	//添加下载数量, 超时, 临时后缀
			this._downloader.setOnTaskError(function(sender, errorCode, errorCodeInternal, errorStr){
				if( errorCodeInternal != 0 && errorCodeInternal != -3 )	// 超时不上报
					zz.Error( sender.requestURL + "  errorCode:" + errorCode + "  " + errorCodeInternal + "  down error:" + errorStr );
				else
					cc.error( sender.requestURL + "  errorCode:" + errorCode + "  " + errorCodeInternal + "  down error:" + errorStr );

				//失败直接删掉
				var tempPath = sender.storagePath + ".tmp";
				if( jsb.fileUtils.isFileExist(tempPath) ){
					jsb.fileUtils.removeFile(tempPath);
				}
				tempPath = sender.storagePath + ".temp";
				if( jsb.fileUtils.isFileExist(tempPath) ){
					jsb.fileUtils.removeFile(tempPath);
				}

				cbArr = self._downloadCallback[sender.storagePath];
				if( cbArr ){
					for( var i = 0; i < cbArr.length; i++ ){
						cbArr[i]();
					}
				}
				else{
					cc.error("******************** there is no cb in '_downloadCallback' " + sender.storagePath + " ***************************");
				}
				delete self._downloadCallback[sender.storagePath];

				// //TODO start jaye 调整加载限制
				// self._tLoadCount--;
				// if(self._tCDNLoadArr.length > 0){
				// 	var data = self._tCDNLoadArr.shift();
				// 	self.LoadCdnFile(data.url,data.cb);
				// }
				// //TODO end jaye 调整加载限制
			});
			this._downloader.setOnFileTaskSuccess(function(sender){
				if( hasOldFile )	//有旧文件得清空下fullPath的缓存 不然新的不会生效
					jsb.fileUtils.purgeCachedEntries();
				cbArr = self._downloadCallback[sender.storagePath];
				if( cbArr ){
					for( var i = 0; i < cbArr.length; i++ ){
						cbArr[i]();
					}
				}
				else{
					cc.error("******************** there is no cb in '_downloadCallback' " + sender.storagePath + " ***************************");
				}
				delete self._downloadCallback[sender.storagePath];

				// //TODO start jaye 调整加载限制
				// self._tLoadCount--;
				// if(self._tCDNLoadArr.length > 0){
				// 	var data = self._tCDNLoadArr.shift();
				// 	self.LoadCdnFile(data.url,data.cb);
				// }
				// //TODO end jaye 调整加载限制
			});
		}
        // 针对某些系统不会自动补全创建路径
        var index = Math.max(writePath.lastIndexOf("/"),writePath.lastIndexOf("\\"));
        var dir = index > -1 ? writePath.substring(0,index) : null;
        if(dir) jsb.fileUtils.createDirectory(dir)
		this._downloader.createDownloadFileTask( cdnUrl, writePath );


		// 屏蔽HttpRequest版
		// var cdnUrl = GameConfig.WEB_CDN_URL + url;
		// var xhr = cc.loader.getXMLHttpRequest();
		// xhr.open("GET", cdnUrl, true);
		// xhr.setRequestHeader("Accept-Charset", "x-user-defined");
		// var self = this;
		// xhr.responseType = "arraybuffer";
		// xhr.onload = function () {
		// 	if( xhr.readyState === 4 && xhr.status === 200 ){
		// 		var bytes = new Uint8Array(xhr.response);
		//
		// 		var writePath = jsb.fileUtils.getWritablePath() + GameConfig.downloadPath + url;		//写用绝对路径
		// 		var index = writePath.lastIndexOf("/");
		// 		var dir = writePath.substring( 0, index );
		// 		if( !jsb.fileUtils.isDirectoryExist(dir) )
		// 		{
		// 			jsb.fileUtils.createDirectory( dir );
		// 		}
		// 		var isSuccess = jsb.fileUtils.writeDataToFile(bytes, writePath);
		// 		isSuccess || cc.log( "Write File Failed   write path: " + jsb.fileUtils.getWritablePath() + " url: " + url );
		// 		cb();
		// 	}
		// 	else{
		// 		cc.log("Load Error: " + cdnUrl);
		// 		cb();	//这里就不处理了 直接回调吧 里面也会err
		// 	}
		// };
		// xhr.send();
	},

	Str2Uint8Array:function (strData) {
		if (!strData)
			return null;

		var arrData = new Uint8Array(strData.length);
		for (var i = 0; i < strData.length; i++) {
			arrData[i] = strData.charCodeAt(i) & 0xff;
		}
		return arrData;
	},

	/**
	 *	清理缓存
	 */
	ClearCache:function(){
		jsb.fileUtils.removeDirectory(jsb.fileUtils.getWritablePath() + GameConfig.storagePath);
		jsb.fileUtils.removeDirectory(jsb.fileUtils.getWritablePath() + "update_temp/");    //temp 也要移除才行
		jsb.fileUtils.purgeCachedEntries();
	}
};