﻿/**
 * 范围音频，在一个圆形区域内音量会根据距离变化
 * pbk 2015年11月5日
 */
var RoundSound = cc.Class.extend({
	/**
	 * 参数1：loop, x, y, radius
	 * 参数2：loop, pos, radius
	 */
	ctor:function( loop, x, y, radius ){
		var self = this;

		if ( arguments.length == 4 ){
			self.bLoop = loop;
			self.SetBindPos( arguments[1], arguments[2] );
			self.SetSoundRadius( arguments[3] );
		} else if ( arguments.length == 3 ){
			self.bLoop = loop;
			self.SetBindPos( arguments[1] );
			self.SetSoundRadius( arguments[2] );
		} else {
			self.bLoop = false;
		}

		self._nDx = Number.MAX_VALUE;// 焦点间距X 
		self._nDy = Number.MAX_VALUE;// 焦点间距Y
		self._nMaxGradientArea = 20 * MGameDefine.GRID_WIDTH;// 音量最大渐变区域
		self.bDispose = false;// 当前音频是否销毁
	},

	/**
	 * 音频ID
	 */
	GetID:function(){
		return this.audio ? this.audio.id : -1;
	},

	/**
	 * 设置音频
	 */
	SetAudio:function( audio ){
		this.audio = audio;
	},

	/**
	 * 设置绑定位置
	 * 参数1：int(x)和int(y) 可用于静态音频
	 * 参数2：object（包含x和y两个属性） 可用于动态音频
	 */
	SetBindPos:function(){
		if ( arguments.length == 2 ){
			if ( typeof(arguments[0]) == "number" 
				&& typeof(arguments[1]) == "number" )
			{
				this._oBindPos = {x:arguments[0], y:arguments[1]};
			}
		} else if ( arguments.length == 1 ){
			if ( typeof(arguments[0]) == "object" 
				&& arguments[0].hasOwnProperty("x") 
				&& arguments[0].hasOwnProperty("y") )
			{
				this._oBindPos = arguments[0];
			}
		}
	},

	/**
	 * 音频有效半径
	 * @param radius
	 */
	SetSoundRadius:function( radius ){
		this._nSoundRadius = radius;
	},

	/**
	 * 更新音量
	 * @param focus 参数为Null情况在外部检测
	 */
	UpdateVolume:function( focus ){
		var self = this;

		if ( !self.audio )
			return;

		var dx = self._oBindPos.x - focus.x;
		var dy = self._oBindPos.y - Math.abs( focus.y );
		if ( dx == self._nDx && dy == self._nDy ){
			return;
		} else {
			self._nDx = dx;
			self._nDy = dy;
		}

		var audio = self.audio;
		var dis = Math.sqrt( dx * dx + dy * dy );
		if ( dis > self._nSoundRadius ){
			// 离开音频区域
			if ( audio.GetPlaying() ){
				audio.Stop();
//				zz.Log( "离开音频区，停止播放" );
			}
		} else {
			// 在有效区内
			if ( !audio.GetPlaying() ){
				audio.Play( 0, self.bLoop );
//				zz.Log( "进入音频区，开始播放" );
			}

			// 离边界距离 / 实际最大渐变区域
			var vol = (self._nSoundRadius - dis) / (self._nSoundRadius > self._nMaxGradientArea ? self._nMaxGradientArea : self._nSoundRadius);

			if ( vol != audio.GetVolume() ){
//				zz.Log( "音量：" + vol + " 最大区域：" + self._nMaxGradientArea + " 音效区域：" + self._nSoundRadius + " 距离：" + dis + " " + dx + " " + dy );
				audio.SetVolume( vol );
			}
		}
	},

	Dispose:function(){
		var self = this;

		self.audio = null;
		self._oBindPos = null;
		self._nSoundRadius = null;
		self._nDx = null;
		self._nDy = null;

		self.bDispose = true;
	}
});

/**
 * 轨迹音效，会控制左右声道的变化
 * pbk 2015年11月5日
 */
var TraceSound = RoundSound.extend({
	
	/**
	 * 更新音量
	 * @param focus 参数为Null情况在外部检测
	 */
	UpdateVolume:function( focus ){
		var self = this;

		if ( !self.audio )
			return;

		var dx = self._oBindPos.x - focus.x;
		var dy = self._oBindPos.y - Math.abs( focus.y );
		if ( dx == self._nDx && dy == self._nDy ){
			return;
		} else {
			self._nDx = dx;
			self._nDy = dy;
		}

		var audio = self.audio;
		var dis = Math.sqrt( dx * dx + dy * dy );
		if ( dis > self._nSoundRadius ){
			// 离开音频区域
			if ( audio.GetPlaying() ){
				audio.Stop();
//				zz.Log( "离开音频区，停止播放" );
			}
		} else {
			// 在有效区内
			// 离边界距离 / 实际最大渐变区域
			var vol = (self._nSoundRadius - dis) / (self._nSoundRadius > self._nMaxGradientArea ? self._nMaxGradientArea : self._nSoundRadius);

//			zz.Log( dx > 0 ? "右音道" : "左音道" );

			if ( dx > 0 ){
				// 音频在焦点右边，左边降音0.3
				if ( vol != audio.GetRightVolume() ){
					audio.SetVolume( 0.05/*vol * 0.3*/, vol );
				}
			} else {
				// 音频在焦点左边，右边降音0.3
				if ( vol != audio.GetLeftVolume() ){
					audio.SetVolume( vol, 0.05/*vol * 0.3*/ );
				}
			}
			
			if ( !audio.GetPlaying() ){
				audio.Play( 0, self.bLoop );
//				zz.Log( "进入音频区，开始播放" );
			}
		}
	},
});
