﻿/** 类型：范围音频*/
var SoundType_Round = SoundType_Round || 1;
/** 类型：轨迹音频*/
var SoundType_Trace = SoundType_Trace || 2;

/**
 * 音频引擎
 * 效果1：音频绑定位置，根据与主玩的距离调节音频大小，音频循环播放（渐弱渐强）
 * 效果2：根据音频在相对于主玩的左右位置，分别使用左右声道播放（弹道从左到右时声效效果）
 * 效果3：发音轨数限制，网页32位
 * pbk 2015年11月4日
 */
var SoundMgr = cc.Class.extend({
	ctor:function(){
		var self = this;
		self._iMaxChannel = 32;// 最大的音频通道
		self._toId = 1;// 音频ID分配枚举
		self._bMute = false;// 是否静音

		self._bindFoucs = null;// 绑定焦点对象

		self._dynaSoundList = [];// 动态音频列表

		self._soundMap = {};// 音频资源列表
		self._soundIdMap = {};// 音频资源ID列表
		self._bgm = null;

		self._operateClean = [];// 操作记录：清理
		self._operateStop = [];// 操作记录：停止

		self.bgmVolume = 0.5;
		self.soundVolume = 0.5;
		self.bgmState = 1;

		self.StartAutoAdjust();
	},

	// ====================================================================================================
	// ====================================================================================== 基础操纵接口
	// ====================================================================================================
	/**
	 * 停止音频（注意：暂不适用于动态音频）
	 * @param id
	 */
	StopSound:function( id ){
        var audio = this._soundIdMap[id];
		if ( audio ){
			audio.Stop();
		} else {
			this._operateStop.push( id );
		}
	},

	/**
	 * 播放音频（注意：暂不适用于动态音频）
	 * @param id
	 */
	PlaySoundByID:function( id ){
		var audio = this._soundIdMap[id];
		if ( audio ){
			audio.Play( 0 );
		} else {
			DelArrayElement( this._operateStop, id );
		}
	},

	/**
	 * 暂停音频（注意：暂不适用于动态音频）
	 * @param id
	 */
	PauseSound:function( id ){
		var audio = this._soundIdMap[id];
		if ( audio ){
			audio.Pause();
		} else {
			this._operateStop.push( id );
		}
	},

	/**
	 * 恢复音频（注意：暂不适用于动态音频）
	 * @param id
	 */
	ResumeSound:function( id ){
		var audio = this._soundIdMap[id];
		if ( audio ){
			audio.Resume();
		} else {
			DelArrayElement( this._operateStop, id );
		}
	},

	/**
	 * 重新播放音频（注意：暂不适用于动态音频）
	 * @param id
	 */
	RewindSound:function( id ){
		var audio = this._soundIdMap[id];
		if ( audio ){
			audio.Stop();
			audio.Play();
		}
	},
	/**
	 * 设置音量
	 */
	SetVolume:function(id,volume)
	{
		var audio = this._soundIdMap[id];
		if ( audio ){
			audio.SetVolume(volume);
		}
	},
	StartMusic:function(value){
		this.bgmState = value ? 1: 0;
		this.SetBackgroundVolume( this.bgmVolume );
	},
	StartSound:function(value){
		this._isPlaySound = value;
		if( !value ){
			this.StopAllSound();
		}
	},
	/**
	 * 设置背景音乐音量(value 0-1)
     */
	SetBackgroundVolume:function(value)
	{
		value = value > 1 ? value/100 : value;
		this.bgmVolume = value;
		if( this._bgm )
			this._bgm.SetVolume(value * this.bgmState);

	},
	/**
	 * 设置音效音量(value 0-1)
	 */
	SetEffectVolume:function(value)
	{
		value = value > 1 ? value/100 : value;
		this.soundVolume = value;

		var map = this._soundMap;
		var list = null;
		var idx = null;
		for ( var url in map ){
			list = map[url];
			for ( idx = 0; idx < list.length; ++idx ){
				list[idx].SetVolume(value);
			}
		}
	},
	/**
	 * 获取音量
	 */
	GetVolume:function(id)
	{
		var audio = this._soundIdMap[id];
		if ( audio ){
			return audio.GetVolume();
		}
		return this.soundVolume;
	},
	/**
	 * 播放所有音频
	 */
	PlayAllSound:function(){
		var map = this._soundIdMap;
		for ( var id in map ){
			map[id].Play( 0 );
		}

		this._bMute = true;
	},

	/**
	 * 停止所有音频
	 */
	StopAllSound:function(){
        this._DelAllSound(false);
	},

	// ====================================================================================================
	// ====================================================================================== 音频资源管理
	// ====================================================================================================
	// 添加音频资源
	_AddMusic:function( audio ){
		if( this._bgm )
			this._bgm.Stop();
		this._bgm = audio;

	},
	_AddSound:function( audio ){
		var list = this._soundMap[audio.src];
		if ( !list ){
			list = [];
			this._soundMap[audio.src] = list;
		}
		list.push( audio );
		this._soundIdMap[audio.id] = audio;
	},

	// 删除音频资源
	_DelSound:function( audio ){
		if( audio ){
			audio.Stop();

			var list = this._soundMap[audio.src];
			if ( list ){
				DelArrayElement( list, audio );

				if ( list.length === 0 ){
					delete this._soundMap[audio.src];
				}
			}

			delete this._soundIdMap[audio.id];
		}
	},

	// 删除所有资源
	_DelAllSound:function(containsBGM){
		containsBGM = containsBGM != false;
		var self = this;
		var map = self._soundMap;
		var list = null;
		var idx = null;
		for ( var url in map ){
			list = map[url];

			for ( idx = 0; idx < list.length; ++idx ){
				list[idx].Stop();
			}

			list.length = 0;
			delete map[url];
		}

		self._soundIdMap = {};

		self._operateClean.length = 0;
		self._operateStop.length = 0;
		if( containsBGM ){
			if( this._bgm )
				this._bgm.Stop();
			this._bgm = null;
		}
	},

	// ====================================================================================================
	// ====================================================================================== 音频计时检测
	// ====================================================================================================
	/**
	 * 绑定焦点对象（主玩家/视角）
	 * 根据位置调节音量大小
	 */
	SetBindFocus:function( obj ){
		this._bindFoucs = obj;
	},

	/**
	 * 开启自动调节声音
	 */
	StartAutoAdjust:function(){
		if ( this.OpenAuto )
			return;

		this.OpenAuto = TimerUtil.TimeUpdate( this.AutoAdjustDynaSound.bind( this ), this, 0.5 );// 每0.5秒检测一次
	},

	/**
	 * 关闭自动调节声音
	 */
	StopAutoAdjust:function(){
		if ( !this.OpenAuto )
			return;

		TimerUtil.ClearTimeUpdate( this.OpenAuto, this );

		this.OpenAuto = false;
	},

	/**
	 * 自动调节动态音频
	 * 若所有动态音频都是静态音频，则在绑定焦点移动时调用
	 * 若有动态音频是动态音频，则计时器循环调用
	 */
	AutoAdjustDynaSound:function(){
		var self = this;

		if ( self._bMute )
			return;

		if ( !self._bindFoucs )
			return;

		self._MoveTraceSound();
		
		self._dynaSoundList.forEach( function( rs ){
			rs.UpdateVolume( self._bindFoucs );
		} );
	},
	
	// ====================================================================================================
	// ====================================================================================== 动态音频管理
	// ====================================================================================================
	// 初始化动态音频管理，包括会控制“场景音频”的播放
	// 添加动态音频
	_AddDynaSound:function( dynaSound ){
		if ( AddArrayElement( this._dynaSoundList, dynaSound ) ){
			// 添加成功
			if ( this._bindFoucs ){
				dynaSound.UpdateVolume( this._bindFoucs );
			}
		}
	},

	// 删除动态音频
	_DelDynaSound:function( dynaSound ){
		if ( DelArrayElement( this._dynaSoundList, dynaSound ) ){
			// 删除成功
			this._DelSound( dynaSound.audio );
			dynaSound.Dispose();
		}
	},

	// 根据ID删除动态音频
	_DelDynaSoundByID:function( id ){
		var list = this._dynaSoundList;
		var len = list.length;

		for ( var i = 0; i < len; ++i ){
			if ( list[i].GetID() === id ){
				list.splice( i, 1 );
				this._DelSound( list[i].audio );
				list[i].Dispose();
				return true;
			}
		}

		return false;
	},

	// 删除所有动态音频
	_DelAllDynaSound:function(){
		var self = this;

		self._dynaSoundList.forEach( function( rs ){
			self._DelSound( rs.audio );
			rs.Dispose();
		} );

		self._dynaSoundList.length = 0;
	},
	
	// ====================================================================================================
	// ====================================================================================== 测试：轨迹音效
	// ====================================================================================================
	
	_testData :[],
	
	_MoveTraceSound:function(){
		if ( this._testData.length === 0 )
			return;
		
		var list = this._testData;
		var obj = null;
		
		for ( var i = list.length - 1; i >= 0; --i ){
			obj = list[i].obj;
			
			obj.x += 10;
			
			if ( obj.x >= obj.end ){
				this._DelDynaSound( list[i].sound );
				list.splice( i, 1 );
				zz.Log( "idx：" + i + " x：" + obj.x + " delete" );
				break;
			} else {
				zz.Log( "idx：" + i + " x：" + obj.x );
			}
		}
	},
	
	PlayTraceSound:function(){
		var self = this;
		
		if ( !this._bindFoucs )
			return;

		var obj = {x:this._bindFoucs.x - 220, y:-this._bindFoucs.y, end:this._bindFoucs.x + 220};
		
		var sound = new TraceSound( true, obj, 200 );

		LoadMgr.LoadSound( self._sPreUrl + "music/10104.mp3", this, LoadPriority_Preload, 
				{complete:self._LoadSoundComplete, completeParam:{type:SoundType_Trace, sound:sound}} );
		
		self._AddDynaSound( sound );
		
		self._testData.push( {obj:obj, sound:sound} );
	},
});

var tAudio = null;
var tlG = null;
var trG = null;