/**
 * Created by merlin on 2016/12/7.
 */
// eg:
// Profiler.Start("Test InitMainPanel");
// Profiler.Test(function(){EngineMgr.uiMgr.InitMainPanel();}, 100);
// Profiler.Stop("Test InitMainPanel");
// Profiler.ProfilerFun("EngineMgr.uiMgr.InitMainPanel");
// Profiler.ProfilerClass("SceneMainPlayer", window);
// Profiler.ProfilerObject("Mgr.skill");
var Profiler = Profiler || {
        Test:function Test(fun, count){
            count = count || 1;
            var start = TimerUtil.GetTimer();
            var min;
            var max;
            var duration;

            var before = start;
            var after;
            var all = [];
            for( var i = 0; i < count; i++ ){
                fun();
                after = TimerUtil.GetTimer();
                duration = after - before;
                before = after;
                if( i == 0 ){
                    min = duration;
                    max = duration;
                }
                else{
                    min = Math.min( min, duration);
                    max = Math.max( max, duration);
                }
                all.push( duration );
            }
            var end = TimerUtil.GetTimer();
            var cost = end - start;
            var avg = cost / count;
            cc.log( "Profiler Test -------------> " + ToolBase.GetStackTrace(false) + "     total: " + cost + "     count: " + count + "    min: " + min + "    max: " + max + "    avg: " + avg + "     " + parseInt( count / cost * 1000 ) + "/s       all: " + all.join(','));
        },

        _simpleTest:{},
        Start:function(name){
            if( this._simpleTest[name] ){
                cc.error( "Profiler Start -------------> " + name + " is already Running. " );    //简单的就不支持递归了
                return;
            }
            this._simpleTest[name] = TimerUtil.GetTimer();
        },

        Stop:function(name, limit, isLog){
            if( !this._simpleTest[name] ){
                cc.error( "Profiler End -------------> " + name + " does not exist. " );    //简单的就不支持递归了
                return 0;
            }
            limit = limit || 0;
            isLog = isLog != false;
            var cost = TimerUtil.GetTimer() - this._simpleTest[name];
            if( cost > limit && isLog )
                cc.log( "Profiler End -------------> " + name + " cost: " + cost );
            delete this._simpleTest[name];
            return cost;
        },

        /***********************************************************************************
         * 搞成一个简单的调用性能分析   (Function.prototype.call不行)需要手动注册有点蛋疼
         * 反正大概思路就是遍历对象，然后改写方法
         * 先通过手动控制name名 cocos的方法定义自动做堆栈有点蛋疼
         ***********************************************************************************/

        /**
         * 慎用   使用到 instanceof class 判断的地方会出错
         * eg1: Profiler.ProfilerClass("cc.Sprite")
         * eg2: Profiler.ProfilerClass("SceneMainPlayer", window)   最顶层必须加window
         * @param name
         * @param target
         * @constructor
         */
        ProfilerClass:function(name, target){
            var funcName = (name.indexOf(".") > -1 ? name.substring(name.lastIndexOf(".") + 1) : name);
            if ( !target || typeof target != "object" ) {
                target = eval(name.substring(0, name.lastIndexOf(".")));
            }
            var fun = target[funcName];
            var prototype = fun.prototype;
            //改写构造函数和原型中的方法
            this.ProfilerFun(name, target);
            this.ProfilerObject(name + ".prototype", prototype);
        },

        /**
         * eg1: Profiler.ProfilerFun("Mgr.sceneObjMgr.FindObject")
         * eg2: Profiler.ProfilerFun("FindObject", Mgr.sceneObjMgr)
         * @param name  越全越好 便于观测   现在不是打堆栈 直接存的函数名
         * @param target
         * @constructor
         */
        ProfilerFun:function(name, target){
            var funcName = (name.indexOf(".") > -1 ? name.substring(name.lastIndexOf(".") + 1) : name);
            if ( !target || typeof target != "object" ) {
                target = eval(name.substring(0, name.lastIndexOf(".")));
            }
            var fun = target[funcName];
            if ( typeof fun == "function" && !fun.isProfiling ) {
                var proFun = function(){
                    var start = TimerUtil.GetTimer();
                    fun.apply( this, arguments );   //这里是proFun的上下文中 不是ProfilerFun
                    var end = TimerUtil.GetTimer();
                    Profiler.SetProfile( name, end - start );
                }
                proFun.prototype = fun.prototype;
                proFun.isProfiling = true;
                target[funcName] = proFun;
            }
        },

        /**
         * eg1: Profiler.ProfilerObject("Mgr.sceneObjMgr")
         * @param objName
         * @param obj
         * @param recurse
         * @constructor
         */
        ProfilerObject:function( objName, obj, recurse ){
            obj = (obj && typeof target == "object") ? obj : eval(objName);
            if ( obj.isProfiling )
                return;
            obj.isProfiling = true;
            for (var prop in obj) {
                if (typeof obj[prop] == "function") {
                    if (prop != "constructor" && prop != "superclass") {
                        this.ProfilerFun(objName + "." + prop, obj);
                    }
                } else {
                    if (typeof obj[prop] == "object" && recurse) {
                        this.ProfilerObject(objName + "." + prop, obj[prop], recurse);
                    }
                }
            }
        },

        _profileData:{},
        SetProfile:function( name, costTime ){
            var data = this._profileData[name];
            if( !data ){
                data = { count:1, min:costTime, max:costTime, avg:costTime, totalTime:costTime, all:[costTime] };
                this._profileData[name] = data;
            }
            else{
                data.count++;
                data.min = Math.min( data.min, costTime );
                data.max = Math.max( data.max, costTime );
                data.totalTime += costTime;
                data.avg = data.totalTime / data.count;
                data.all.push( costTime );
            }
        },

        /**
         * 取得数据
         * @param sortKey
         * @returns {Array}
         * @constructor
         */
        GetProfile:function( sortKey ){
            sortKey = sortKey || 'totalTime';
            var arr = [];
            for(var key in this._profileData){
                this._profileData[key].key = key;
                arr.push(this._profileData[key]);
            }
            arr.sort(function(a,b){return b[sortKey] - a[sortKey];});
            return arr;
        },

        /**
         * 打印数据
         * @param sortKey
         * @constructor
         */
        LogProfile:function( sortKey ){
            sortKey = sortKey || 'totalTime';
            var arr = this.GetProfile(sortKey);
            var len = arr.length;
            var data;
            for( var i = 0; i < len; i++ ){
                data = arr[i];
                cc.log( "Profiler Log-------------> " + data.key + "      count: " + data.count + "    min: " + data.min + "    max: " + data.max + "    avg: " + data.avg + "     total: " + data.totalTime + "       all: " + data.all.join(','));
            }
        },

        /**
         * 保存起来慢慢查咯
         * @param sortKey
         * @constructor
         */
        SaveProfile:function( sortKey ){
            sortKey = sortKey || 'totalTime';
            var arr = this.GetProfile(sortKey);
            var len = arr.length;
            var str = "";
            var data;
            for( var i = 0; i < len; i++ ){
                data = arr[i];
                str += data.key + "\tcount\t" + data.count + "\tmin\t" + data.min + " \t max\t" + data.max + "\tavg\t" + data.avg + "\ttotal\t" + data.totalTime + "\tall\t" + data.all.join(',') + "\n";
            }
            var url = jsb.fileUtils.getWritablePath() + "profile.txt";
            cc.log( "Profiler Save-------------> " + url );
            jsb.fileUtils.writeStringToFile( str, url );
        },
        /******************              先实现了简单的数据显示    现有数据再丰富一下完全可以实现个Profile UI来直观的检测               ********************/

        GetCurMemory:function(){
            return cc.textureCache.getCachedTextureInfo();
        },

        /************************************    帧间隔性能监测    **************************************/
        BEFORE_UPDATE:1,
        AFTER_UPDATE:2,
        BEFORE_NET:3,
        AFTER_NET:4,
        BEFORE_ONFRAME:5,
        AFTER_ONFRAME:6,
        AFTER_VISIT:7,
        AFTER_DRAW:8,
        _frameObj:{},
        RecordUpdate:function(step){
            if( step == this.BEFORE_NET && this._frameObj[step] )   //消息超出buffer大小 分成多次派发到js 只需要记录一次开始就行了
                return;
            this._frameObj[step] = Date.now();

            if( step == this.AFTER_DRAW )
                this._RocordUpdateEnd();
        },

        _recordList:[],
        _RocordUpdateEnd:function(){
            var all = this._frameObj[this.AFTER_DRAW] - this._frameObj[this.BEFORE_UPDATE];

            if( all > 17 ){
                var net = this._frameObj[this.AFTER_NET] - this._frameObj[this.BEFORE_NET];
                var frame = this._frameObj[this.AFTER_ONFRAME] - this._frameObj[this.BEFORE_ONFRAME];
                var update = this._frameObj[this.AFTER_UPDATE] - this._frameObj[this.BEFORE_UPDATE];
                var visit = this._frameObj[this.AFTER_VISIT] - this._frameObj[this.AFTER_UPDATE];
                var draw = this._frameObj[this.AFTER_DRAW] - this._frameObj[this.AFTER_VISIT];
                var initImg = this._frameObj[this.AFTER_UPDATE] - this._frameObj[this.AFTER_ONFRAME] - net; //net之后 主要是加载后initWithImage
                var action = this._frameObj[this.BEFORE_ONFRAME] - this._frameObj[this.BEFORE_UPDATE];      //onframe之前 主要是action
                this._recordList.push({time:Date.now(), net:net, frame:frame, update:update, visit:visit, draw:draw, all:all, initImg:initImg, action:action, timeout:this._timeoutWasteTime,
                    netList:this._msgRecord.concat(), fight:this._fightBuffer.concat(), timeBuffer:this._timeoutBuffer.concat()});
            }

            for( var i = 1; i <= 8; i++ )
                this._frameObj[i] = 0;

            this._msgRecord.length = 0;
            this._fightBuffer.length = 0;
            this._timeoutBuffer.length = 0;
            this._timeoutWasteTime = 0;
        },
        _startTime:0,
        SubTime:function(isEnd){
            var sub = 0;
            if( isEnd )
                sub = Date.now() - this._startTime;
            else
                this._startTime = Date.now();
            return sub;
        },
        _msgRecord:[],
        _fightBuffer:[],
        _timeoutBuffer:[],
        LogMessage:function(msgId, isEnd){
            var time = this.SubTime(isEnd);
            if( isEnd )
                this._msgRecord.push({id:msgId, time:time});
        },
        LogFight:function(name, isEnd){
            var time = 0;
            if( isEnd )  {
                time = Profiler.Stop(name, 2, false);
                if( time > 1 ){
                    this._fightBuffer.push({name:name, time:time});
                }
            }
            else{
                Profiler.Start(name);
            }
        },

        LogTimeout:function(timeout, isEnd){
            var time = this.SubTime(isEnd);
            if( isEnd && time > 2 )
                this._timeoutBuffer.push( {timeout:timeout, time:time});
            this._timeoutWasteTime += time;
        },

        GetLogStr:function(isReset, isLog){
            var str = "";
            for( var i = 0, len = this._recordList.length; i < len; i++ ){
                var record = this._recordList[i];
                str += record.time + "    all: " + record.all + "    action: " + record.action + "    frame: " + record.frame + "    timeout: " + record.timeout + "    net: " + record.net + "    initImg: " + record.initImg + "    visit: " + record.visit + "    draw: " + record.draw;
                str = this.AddNewLine(str, isLog);
                if( record.net > 15 ){
                    var rLen = record.netList.length;
                    str += "     count: " + rLen;
                    for( var j = 0; j < rLen; j++ ){
                        var net = record.netList[j];
                        str += "  " + net.id + "," + net.time;
                        if( j % 10 == 9 )
                            str = this.AddNewLine(str, isLog);
                    }
                    str = this.AddNewLine(str, isLog);
                }
                var rLen = record.fight.length;
                for( var j = 0; j < rLen; j++ ){
                    var fight = record.fight[j];
                    str += "  " + fight.name + "," + fight.time;
                    if( j % 10 == 9 )
                        str = this.AddNewLine(str, isLog);
                }
                str = this.AddNewLine(str, isLog);
                if( record.timeout > 5 ){
                    var rLen = record.timeBuffer.length;
                    for( var j = 0; j < rLen; j++ ){
                        var timeout = record.timeBuffer[j];
                        str += timeout.timeout._st_name + "  " + timeout.time + "   \n";
                        str += timeout.timeout.callback.original;
                        str = this.AddNewLine(str, isLog);
                    }
                }
            }
            if( isReset )
                this._recordList.length = 0;
            return str;
        },
        AddNewLine:function(str, isLog){
            if( isLog ){
                cc.log(str);
                str = "";
            }
            else{
                str += "\n";
            }
            return str;
        }
    };
