
var EUImpl = ChannelImpl.extend({
    /// 平台
    getPlatform:function(){
        return "eu";
    },
    /// 渠道ID
    getChannelId:function(){
        return this.getPlatform();
    },
    /// 游戏ID
    getGameId:function(){
        return "27"
    },
    /// EU APP ID
    getAppID:function(){
        switch(cc.sys.os){
            case cc.sys.OS_ANDROID:
                jsb.reflection.callStaticMethod("org/cocos2dx/javascript/EuImpl", "getAppID", "()I");
                break;
            case cc.sys.OS_IOS:
                jsb.reflection.callStaticMethod("AppController", "getCurrChannel:");
                break;
            default: return 0;
        }
    },
    /// EU CHANNEL
    getCurrChannel:function(){
        switch(cc.sys.os){
            case cc.sys.OS_WINDOWS: return 0;
            case cc.sys.OS_ANDROID:
                jsb.reflection.callStaticMethod("org/cocos2dx/javascript/EuImpl", "getCurrChannel", "()I");
                break;
            case cc.sys.OS_IOS:
                jsb.reflection.callStaticMethod("AppController", "getCurrChannel:");
                break;
            default: return 0;
        }
    },

    /// 初始化
    init:function(){
        var text = "ChannelUtil.impl.OnSDKCallBack";

        switch(cc.sys.os){
            case cc.sys.OS_WINDOWS: return false;
            case cc.sys.OS_ANDROID:
                jsb.reflection.callStaticMethod("org/cocos2dx/javascript/EuImpl", "init", "(Ljava/lang/String;)V", text);
                break;
            case cc.sys.OS_IOS:
                jsb.reflection.callStaticMethod("EUImpl", "init:",text);
                break;
        }

        return true
    },
    /// 登录
    login:function(){
        if (cc.sys.os == cc.sys.OS_ANDROID) {
            result = jsb.reflection.callStaticMethod("org/cocos2dx/javascript/EuImpl", "login", "()V");
        } else if (cc.sys.os == cc.sys.OS_IOS) {
            result = jsb.reflection.callStaticMethod("EUImpl", "login");
        }
    },
    logout:function(){
        if (cc.sys.os == cc.sys.OS_ANDROID) {
            result = jsb.reflection.callStaticMethod("org/cocos2dx/javascript/EuImpl", "logout", "()V");
        } else if (cc.sys.os == cc.sys.OS_IOS) {
            //result = jsb.reflection.callStaticMethod("EUImpl", "logout");
            EnterGameMgr.Logout();
        }
    },
    /// 退出
    exit:function(){
        if (cc.sys.os == cc.sys.OS_ANDROID) {
            result = jsb.reflection.callStaticMethod("org/cocos2dx/javascript/EuImpl", "exit", "()V");
        } else if (cc.sys.os == cc.sys.OS_IOS) {
            result = jsb.reflection.callStaticMethod("EUImpl", "exit");
        }
    },
    /// 支付
    pay:function(data){
        var params = {
            buyNum			: data.buyNum				,
            coinNum			: data.coinNum				,
            extension		: data.extension			,
            price			: data.price				,
            productId		: data.productId			,
            productName		: data.productName			,
            productDesc		: data.productDesc			,

            orderID			: data.orderID				,

            roleId			: data.roleId				,
            roleLevel		: data.roleLevel			,
            roleName		: data.roleName				,

            serverId		: data.serverId				,
            serverName		: data.serverName			,
            vip				: data.vip	,

            payNotifyUrl	: ""	,
        }

        //// payNotifyUrl 解析出来
        var extension = Base64Util.decode(data.extension).join("")
        extension = extension.replace(";","")
        var ret = {}
        var arr = extension.split("&")
        arr.forEach(v=>{
            let x = v.split("=")
            ret[x[0]] = x[1];
        })
        params["payNotifyUrl"] = ret.payNotifyUrl

        var json = JSON.stringify(params)

        if (cc.sys.os == cc.sys.OS_ANDROID) {
            result = jsb.reflection.callStaticMethod("org/cocos2dx/javascript/EuImpl", "pay", "(Ljava/lang/String;)V", json);
        } else if (cc.sys.os == cc.sys.OS_IOS) {
            result = jsb.reflection.callStaticMethod("EUImpl", "pay:", json);
        }
    },
    /// 游戏登陆后的回调
    onLogin:function(new_role){
        this.SubmitExtraData(new_role ? this.DATA_TYPE.TYPE_CREATE_ROLE : this.DATA_TYPE.TYPE_ENTER_GAME);
    },
    // 升级
    onLevelUp:function(){
        this.SubmitExtraData(this.DATA_TYPE.TYPE_LEVEL_UP);
    },
    /// 游戏选择服务器后的回调
    onServerSelected:function(){
        this.SubmitExtraData(this.DATA_TYPE.TYPE_SELECT_SERVER);
    },

    OnSDKCallBack:function(type,data){
        cc.log("=============1", type)
        cc.log("=============2", data)
        cc.log("=============2", data.uToken)
        switch (type) {
            case "onInitResult":
                this.util.OnSDK_Init(0,data)
                break;
            case "onLoginResult":
                let uToken = data

                let params = {}
                params.userID = uToken.userID;
                params.username = uToken.username;
                params.sdkUserID = uToken.sdkUserID;
                params.sdkUsername = uToken.sdkUsername;
                params.timestamp = uToken.timestamp;
                params.extension = uToken.extension;
                params.token = uToken.token

                this.util.OnSDK_Login(0,params)
                break;
            case "onSwitchAccount":
                this.util.OnSDK_SwitchAccount(0,data)
                break;
            case "onLogout":
                /// 稍作延迟，即时弹出会导致按钮重叠
                TimerUtil.SetTimeout(this,function(){
                    this.util.OnSDK_Logout(0,data)
                },100)
                break;
            case "onExit":
                AlertExt.Show( "你确定要退出游戏吗？", TipsEnum.BUTTON_SURE_AND_FALSE, {
                    _onFunc:function(){
                        cc.game.end();
                    },
                });
                break;
        }
    },
    DATA_TYPE : {
        TYPE_SELECT_SERVER	: 1	,//选择服务器
        TYPE_CREATE_ROLE	: 2	,//创建角色
        TYPE_ENTER_GAME		: 3	,//进入游戏
        TYPE_LEVEL_UP		: 4	,//等级提升
        TYPE_EXIT_GAME		: 5	,//退出游戏
    },
    SubmitExtraData:function(dataType){
        var data = {};
        data["dataType"]	= dataType;
        data["moneyNum"]	= this.GetMoney();
        data["roleID"]		= this.GetRoleId();
        data["roleName"]	= this.GetRoleName();
        data["roleLevel"]	= this.GetRoleLevel();
        data["serverId"]	= this.GetServerId();
        data["serverName"]	= this.GetServerName();
        data["vip"]			= this.GetVipLevel();

        ///
        var json = JSON.stringify(data)
        if (cc.sys.os == cc.sys.OS_ANDROID) {
            jsb.reflection.callStaticMethod("org/cocos2dx/javascript/EuImpl", "submitExtraData", "(Ljava/lang/String;)V", json);
        } else if (cc.sys.os == cc.sys.OS_IOS) {
            jsb.reflection.callStaticMethod("EUImpl", "submitExtraData:", json);
        }
    },
})